package sensorComponent;

import gamf.framework.GenericAutonomicManagementFramework;
import gamf.interfaces.framework.IGAMF;
import gamf.interfaces.framework.IManagedFacet;
import gamf.interfaces.systemAdapter.IEffector;
import gamf.interfaces.systemAdapter.IEventGenerator;
import gamf.interfaces.systemAdapter.IMetricExtractor;
import gamf.interfaces.systemAdapter.IPolicyEvaluator;

public class Manager {

	IEffector mySensorComponentEffector;
	IEventGenerator mySensorComponentEventGenerator;

	public Manager(IEffector mySensorComponentEffector,
			IEventGenerator mySensorComponentEventGenerator) throws Exception {

		// NOTE: all classes starting with "My" have to be provided by the
		// system adapter programmer (i.e. you)

		// instantiate a manager
		IGAMF gamf = new GenericAutonomicManagementFramework();

		// specify the facet
		IManagedFacet managedFacet = gamf
				.registerManagedFacet(Constants.MANAGED_FACET);

		// create your event generator
		this.mySensorComponentEventGenerator = mySensorComponentEventGenerator;
		// register your event generator with the GAMF
		gamf.add(mySensorComponentEventGenerator, managedFacet);

		// create your effector
		this.mySensorComponentEffector = mySensorComponentEffector;
		// register your effector with the GAMF
		gamf.add(mySensorComponentEffector, managedFacet);

		// create your metric extractor
		IMetricExtractor mySensorComponentMetricExtractor = new MySensorComponentMetricExtractor();
		// register your metric extractor with the GAMF
		gamf.add(mySensorComponentMetricExtractor, managedFacet);

		// create your policy evaluator
		IPolicyEvaluator mySensorComponentPolicyEvaluator = new MySensorComponentPolicyEvaluator(
				mySensorComponentMetricExtractor);
		mySensorComponentPolicyEvaluator
				.associateEffectorWithPolicyEvaluator(mySensorComponentEffector);
		// register your policy evaluator with the GAMF
		int policyEvaluationInterval = Constants.OBSERVATION_PERIOD; // in ms
		gamf.add(mySensorComponentPolicyEvaluator,
				policyEvaluationInterval, managedFacet);
	}

	IEffector getEffector() {
		return mySensorComponentEffector;
	}

	IEventGenerator getEventGenerator() {
		return mySensorComponentEventGenerator;
	}

}
