package sensorComponent;

import uk.ac.standrews.cs.nds.util.Diagnostic;
import gamf.addons.apstractSystemAdapter.PolicyEvaluator;
import gamf.framework.exceptions.MetricExtractionException;
import gamf.framework.exceptions.PolicyEvaluationException;
import gamf.framework.exceptions.TypeMissMatch;
import gamf.interfaces.framework.IMetricResult;
import gamf.interfaces.systemAdapter.IEffector;
import gamf.interfaces.systemAdapter.IMetricExtractor;
import gamf.interfaces.systemAdapter.IPolicyEvaluator;

public class MySensorComponentPolicyEvaluator extends PolicyEvaluator implements
		IPolicyEvaluator {

	IMetricExtractor errorMetricExtractor;
	IEffector pollingIntervalEffector;

	public MySensorComponentPolicyEvaluator(
			IMetricExtractor errorMetricExtractor) {
		super(Constants.POLICY_EVALUATOR_ID, Constants.SYSTEM_ADAPTER_CATEGORY);
		this.errorMetricExtractor = errorMetricExtractor;
		
	}

	/**
	 * Determines a new polling interval with respect to the error rate. A error
	 * rate higher than a specific threshold determines an increase; an error
	 * rate smaller than the threshold ({@link Constants.THRESHOLD}) a decrease of the polling interval.
	 * A maximum poling interval of @link {@link Constants.MAX_INTERVAL} is not exceeded
	 */
	public void evaluatePolicy() throws PolicyEvaluationException {
		pollingIntervalEffector = gimmeEffector();
		IMetricResult errorMetric = null;

		try {
			errorMetric = errorMetricExtractor.extractMetricResult();
		} catch (MetricExtractionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		Integer pollingInterval=(Integer)pollingIntervalEffector.get();
		int errorRate = (Integer) errorMetric.getMetricValue();

		if (errorRate >= Constants.THRESHOLD && pollingInterval<=Constants.MAX_INTERVAL) {
			pollingInterval++;
		}else{
			pollingInterval--;
		}
		
		Diagnostic.trace("changing polling interval from:"+pollingIntervalEffector.get()+" to "+pollingInterval);
		try {
			pollingIntervalEffector.set(pollingInterval);
		} catch (TypeMissMatch e) {
			throw new PolicyEvaluationException(e);
		}
	}

	/**
	 * @return the only effector we are using in this example
	 */
	private IEffector gimmeEffector() {
		return effectors.get(0);
	}

}
